/**
 * Deserializes a flat object with serialized keys into a nested object structure.
 * Processes keys that may include delimiters (e.g., ".", "[]") to form nested structures,
 * handling specific key patterns for types like numbers and booleans.
 *
 * @template T
 * @param {T} serialized - An object where keys represent nested paths or types (e.g., "key[0].prop!!").
 * @returns {object} A deserialized object with nested structure and parsed values.
 * @throws {Error} Throws an error if the argument is not an object.
 */
export declare function deserializeObjectKey<T>(serialized: T): object;
/**
 * Serializes a nested object into a flat object with keys representing the path to each value.
 * Supports special suffixes to encode primitive types (e.g., numbers as `key()` and booleans as `key!!`).
 *
 * @template T
 * @param {T} deserialized - An object with nested structure to be serialized into a flat structure.
 * @returns {object} A flat object where each key represents the path to a nested value.
 * @throws {Error} Throws an error if the argument is not an object.
 */
export declare function serializeObjectKey<T>(deserialized: T): object;
/**
 * Normalizes a key by adding a suffix to represent primitive data types.
 * Adds `()` for numbers and `!!` for booleans, leaving other types unchanged.
 *
 * @param {string} key - The key to normalize based on its associated value type.
 * @param {unknown} value - The value associated with the key, used to determine the suffix.
 * @returns {string} The normalized key with a suffix representing its data type, if applicable.
 */
export declare function normalizePrimitiveKey(key: string, value: unknown): string;
/**
 * Removes any type-indicating suffix from a key, such as `()` or `!!`.
 *
 * @param {string} key - The key to denormalize by removing type suffixes, if present.
 * @returns {string} The key without any type-indicating suffix.
 */
export declare function denormalizePrimitiveKey(key: string): string;
/**
 * Parses a value based on a key that may include type indicators.
 * If the key indicates a primitive type (e.g., "number()" or "boolean!!"), it parses the value accordingly.
 * Otherwise, returns the value directly, with special handling for null or undefined conversions.
 *
 * @param {string} key - The key associated with the value, potentially containing a type indicator.
 * @param {string} value - The value to parse, typically a string.
 * @returns {unknown} The parsed value, converted to the indicated type if applicable.
 */
export declare function valueParserByKey(key: string, value: string): unknown;
